@@ -7,18 +7,20 @@
 #'
 #' @param md Character.  A vector of character strings which contains the metadata.
 #'
-#' @param mode Character. One of c("IR_etc", "NMR", "NMR2D").
+#' @param mode Character. One of c("IR_etc", "NMR", "NMR2D").  Note the same as "fmt" (see notes in code).
+#'        Determines the needed processing.
 #'
 #' @param SOFC Logical. Stop on Failed Check.   See \code{\link{readJDX}} for details.
 #'
 #' @param debug Integer.  See \code{\link{readJDX}} for details.
 #'
 #' @return A named numeric vector containing the extracted parameters.
-#' Contents will vary by \code{mode}.
+#'         Contents will vary by \code{mode}.
 #'
 #' @noRd
 #'
 extractParams <- function(md, mode, SOFC, debug = 0) {
+
   if (mode == "XY_data") {
 
     # The following parameters must be found
@@ -205,17 +207,72 @@ extractParams <- function(md, mode, SOFC, debug = 0) {
     }
   } # end of mode == "NMR_2D"
 
+  if (mode == "LC_MS") {
+
+    # This section does NOT currently make the EU conversion; watch out for strsplit choice
+    # No parameters in this section can be skipped via SOFC
+
+    # There do not appear to be any officially mandated checks, but we'll try to make some anyway,
+    # but use the SOFC mechanism
+
+    # TODO We are not currently checking any of these values; possibly the same in NMR_2D above
+
+    npoints <- grep("^\\s*##VAR_DIM\\s*=", md) # in LC-MS this is the number of time points
+    if (SOFC) if (npoints == 0L) stop("Couldn't find VAR_DIM")
+    if (npoints != 0L) {
+      npoints <- md[npoints]
+      npoints <- sub("^\\s*##VAR_DIM\\s*=", replacement = "", npoints)
+      npoints <- as.numeric(unlist(strsplit(npoints, ",")))
+    }
+
+    firsts <- grep("^\\s*##FIRST\\s*=", md)
+    if (SOFC) if (length(firsts) == 0) stop("Couldn't find FIRST")
+    if (firsts != 0L) {
+      firsts <- md[firsts]
+      firsts <- sub("^\\s*##FIRST\\s*=", replacement = "", firsts)
+      firsts <- as.numeric(unlist(strsplit(firsts, ",")))
+    }
+
+    lasts <- grep("^\\s*##LAST\\s*=", md)
+    if (SOFC) if (lasts == 0) stop("Couldn't find LAST")
+    if (lasts != 0L) {
+      lasts <- md[lasts]
+      lasts <- sub("^\\s*##LAST\\s*=", replacement = "", lasts)
+      lasts <- as.numeric(unlist(strsplit(lasts, ",")))
+    }
+
+    # The following assumes Waters Acquity QDA which exports the first and
+    # last values in the wrong order (they refer to time, but are in the intensity position in the vector)
+    # In the future, may need to extract vendor and use that info here
+    time_points <- npoints[2]
+    first_time <- firsts[3]
+    last_time <- lasts[3]
+
+    # if !SOFC these values will be zero
+    params <- c(
+      as.numeric(time_points), as.numeric(first_time), as.numeric(last_time)
+    )
+    names(params) <- c("time_points", "first_time", "last_time")
+
+    if (debug == 2) {
+      cat("\nExtracted parameters:\n")
+      print(params)
+    }
+  } # end of mode == "LC_MS"
+
   if (mode == "PEAK_TABLE") {
 
     # There are no official checks for this format
 
     npoints <- grep("^\\s*##NPOINTS\\s*=", md)
-    if (npoints == 0) stop("Couldn't find NPOINTS")
-    npoints <- md[npoints]
-    npoints <- sub("^\\s*##NPOINTS\\s*=", replacement = "", npoints)
-    npoints <- as.integer(npoints)
+    if (SOFC) if (npoints == 0) stop("Couldn't find NPOINTS")
+    if (npoints != 0L) {
+      npoints <- md[npoints]
+      npoints <- sub("^\\s*##NPOINTS\\s*=", replacement = "", npoints)
+      npoints <- as.integer(npoints)
+    }
 
-    params <- npoints
+    params <- npoints # if !SOFC this will be 0L
     names(params) <- "npoints"
 
     if (debug == 2) {
