@@ -15,22 +15,22 @@
 #' The default is \code{TRUE} i.e. stop when something is not right.
 #' This ensures that correct data is returned.  Change to \code{FALSE} at your own risk.
 #' NOTE: Only certain checks can be skipped via this option, as there are some
-#' parameters that must be available and correct in order to return any answer.
+#' parameters that must be available and correct in order to return \emph{any} answer.
 #' For instance, one must end up with the same number of X and Y values.
 #' This option is provided for those \pkg{advanced
 #' users} who have carefully checked their original files and want to skip the
 #' required checks.  It may also be useful for troubleshooting.
-#' The JCAMP-DX standard requires
+#' The JCAMP-DX standard typically requires
 #' several checks of the data as it is decompressed.  These checks are essential
 #' to obtaining the correct results.  However, some JCAMP-DX writing programs
 #' do not follow the standard to the letter.  For instance we have observed that
 #' not all JCAMP-DX writers put FIRSTY into the metadata, even though it is required by
 #' the standard.  In other cases values in the file have low precision (see section on precision).
-#' Another example is we have observed files where the X values are the count of data points,
-#' and FIRSTY is given in Hz.  Since the field strength and center of the sweep frequency is needed
-#' to convert to ppm, and these are  not required in the standard, one cannot return an answer in
-#' ppm or Hz automatically.
-#' In cases like this, once can set \code{SOFC = FALSE} and then manually convert the X axis.
+#' Another example we have observed is NMR files where the X values are the count/index of data points,
+#' and FIRSTY is given in Hz.  Since the field strength and center of the sweep frequency are needed
+#' to convert to ppm, and these are parameters not required in the standard, one cannot return an
+#' answer in either ppm or Hz automatically.
+#' In cases like this, one can set \code{SOFC = FALSE} and then manually convert the X axis.
 #'
 #' @param debug Integer.  The level of debug reporting desired.  For those options giving
 #'        a lot of output, you may wish to consider directing the output via \code{\link{sinkall}}
@@ -44,62 +44,58 @@
 #'   \item 5 = detailed info about processing the Y values when DUP is in use (huge output!).
 #'   \item 6 = detailed info about processing the Y values when DIF is in use (huge output!).
 #' }
-#' In cases where an error is about to stop execution, you get additional information regardless of
+#' In cases when an error is about to stop execution, you get additional information regardless of
 #' the \code{debug} value.
 #'
 #' @return A list, as follows:
 #'
 #' \itemize{
-#'
-#' \item The first element is a data frame summarizing the pieces of the imported file.
-#'
-#' \item The second element is the file metadata.
-#'
-#' \item The third element is a integer vector giving the comment lines found
+#'   \item The first element is a data frame summarizing the pieces of the imported file.
+#'   \item The second element is the file metadata.
+#'   \item The third element is a integer vector giving the comment lines found
 #'       (exclusive of the metdata, which typically contains many comments).
 #' }
 #'
 #' Additional elements contain the extracted data as follows:
 #'
 #' \itemize{
-#'
-#' \item If the file contains multiple spectra
-#' (not currently supported), there will be one data frame for each spectrum.
-#'
-#' \item If the file contains the real and imaginary
-#' parts of a 1D NMR spectrum, there will be two data frames, one containing the real portion
-#' and the other the imaginary portion.
-#'
-#' \item If the file contains one non-NMR spectrum,
-#' a single data frame will be returned.
-#'
-#' \item In all cases above, the data frame has
-#' elements \code{x} and \code{y}.
-#'
-#' \item In the case of 2D NMR data, additional list elements are returned including
-#' the F2 frequency values, the F1 frequency values, and a matrix containing the 2D data.
-#'
+#'   \item If the file contains one non-NMR spectrum, or a processed NMR spectrum (i.e. only 
+#'         the final real data), a single data frame.
+#'   \item If the file contains the real and imaginary
+#'          parts of a 1D NMR spectrum, there will be two data frames, one containing the real portion
+#'          and the other the imaginary portion.
+#'   \item In all cases above, the data frame has elements \code{x} and \code{y}.
+#'   \item In the case of 2D NMR data, additional named list elements are returned including
+#'         the F2 frequency values, the F1 frequency values, and a matrix containing the 2D data.
+#'   \item In the case of LC-MS or GC-MS data, a data frame is returned for each time point.
+#'         The data frame has elements \code{mz} and \code{int} (intensity). Each time point
+#'         is named with the time from the file.
 #' }
 #'
 #' @seealso Do \code{browseVignettes("readJDX")} for background information,
-#' references, supported formats, and details about the roles of each function.
+#'          references, supported formats, and details about the roles of each function.
+#'          If you have a multiblock file (which contains multiple spectra, but not 2D NMR,
+#'          LC-MS or GC-MS data sets), please see
+#'          \code{\link{splitMultiblockDX}} for a function to break such files into
+#'          individual files which can then be processed in the normal way.
 #'
 #' @section Included Data Files:
-#' The examples make use of data files included with the package. File \code{SBO.jdx}
-#' is an IR spectrum of Smart Balance Original spread (a butter substitute). The
-#' spectrum is presented in transmission format, and was recorded on a ThermoFisher
-#' instrument.  The file uses AFFN compression, and was written
-#' with the JCAMP-DX 5.01 standard. Note that even though the Y-axis was recorded in
-#' percent transmission, in the JDX file it is stored on [0\ldots1].
-#' File \code{PCRF.jdx} is a 1H NMR
-#' spectrum of a hexane extract of a reduced fat potato chip.  The spectrum was
-#' recorded on a JEOL instrument.  The file uses SQZ DIF DUP compression, and was written
-#' with the JCAMP-DX 6.00 standard.
-#' File \code{PCRF_line265.jdx} has a deliberate error in it.
-#' File \code{isasspc1.jdx} is a 2D NMR file recorded on a JEOL GX 400 instrument.
-#' The file is freely available at \url{http://www.jcamp-dx.org/}.
-#' File \code{MiniDIFDUP.JDX} is a small demonstration file, used in the vignettes to
-#' illustrate the decompression process.  It is derived from a freely available file.
+#' The examples make use of data files included with the package:
+#' \itemize{
+#'   \item File \code{SBO.jdx} is an IR spectrum of Smart Balance Original spread (a butter
+#'         substitute). The spectrum is presented in transmission format, and was recorded on a
+#'         ThermoFisher instrument.  The file uses AFFN compression, and was written
+#'          with the JCAMP-DX 5.01 standard. Note that even though the Y-axis was recorded in
+#'          percent transmission, in the JDX file it is stored on [0\ldots1].
+#'   \item File \code{PCRF.jdx} is a 1H NMR spectrum of a hexane extract of a reduced fat potato chip.
+#'         The spectrum was recorded on a JEOL instrument.  The file uses SQZ DIF DUP compression,
+#'         and was written with the JCAMP-DX 6.00 standard.
+#'   \item File \code{PCRF_line265.jdx} has a deliberate error in it.
+#'   \item File \code{isasspc1.jdx} is a 2D NMR file recorded on a JEOL GX 400 instrument.
+#'         The file is freely available at \url{http://www.jcamp-dx.org/}.
+#'   \item File \code{MiniDIFDUP.JDX} is a small demonstration file, used in the vignettes to
+#'         illustrate the decompression process.  It is derived from a freely available file.
+#' }
 #'
 #' @section Precision:
 #' Internally, this package uses a tolerance factor when comparing values during certain checks.
@@ -114,8 +110,8 @@
 #' @section Y Value Check:
 #' The standard requires a "Y Value Check" when in DIF mode.  Extra Y values have been appended to each
 #' line to use in the check, and the last Y value on a line must equal the first Y value on the next line
-#' IFF one is in DIF mode.  After a successful check, the extra Y value must be removed.  In actual practice,
-#' some vendors, at least some of the time, seem to differ as to the definition of
+#' \emph{IFF} one is in DIF mode.  After a successful check, the extra Y value must be removed.  In actual practice,
+#' some vendors, at least some of the time, seem to differ as to the meaning of
 #' "being in DIF mode".  In turn, this determines how the Y value check should proceed.
 #' \itemize{
 #'   \item The standard says "When, and only when, the last ordinate of a line is in DIF form ...
@@ -128,7 +124,7 @@
 #'         for Y value check purposes. In these cases we have look backwards to see if we are in DIF mode.
 #'         Let's call this "relayed DIF".
 #'   \item However, some vendors may treat ... DIF DUP DUP (end of line) as not in DIF mode, and hence
-#'         one should not do the Y value check and not remove any values, since this vendor would not have
+#'         one should not do the Y value check and not remove any values, as this vendor would not have
 #'         added an extra Y value.
 #'   \item In addition to these three possibilities, \code{readJDX} through versions 0.3.xx used a different
 #'         definition, namely if there were any DIF entries anywhere on the line, then DIF mode was
@@ -184,6 +180,7 @@
 #' }
 #'
 readJDX <- function(file = "", SOFC = TRUE, debug = 0) {
+
   if (!requireNamespace("stringr", quietly = TRUE)) {
     stop("You need to install package stringr to use this function")
   }
@@ -196,29 +193,30 @@ readJDX <- function(file = "", SOFC = TRUE, debug = 0) {
 
   # A data block consists of ##TITLE= up to ##END=
   # However, link blocks can be used to contain data blocks, in which
-  # case one has a compound file. Link blocks and compound files are not supported.
-  # NMR data sets, including 2D NMR data sets, use a different scheme to hold multiple data sets.
+  # case one has a compound file. Link blocks and compound files are not supported, 
+  # but a function is available to split them into individual files.
+  # NMR data sets, including 2D NMR data sets, and LC-MS/GC-MS data sets use a different
+  # scheme (NTUPLES) to hold multiple data sets.
 
   blocks <- grep("^\\s*##TITLE\\s*=.*", jdx)
   nb <- length(blocks)
   if (nb == 0) stop("This does not appear to be a JCAMP-DX file")
-  if (nb > 1) stop("Compound (multi-block / multi-spectra) data sets not supported")
+  if (nb > 1) stop("Compound (multi-block / multi-spectra) data sets can not be parsed.\nSee splitMultiblockDX which is a function to split such files into separate files which can be parsed.")
 
   ##### Step 2. Locate the parameters and the variable list(s)
 
   VL <- findVariableLists(jdx, debug)
 
-  # fmt is a character vector extracted by findVariableLists.
+  # "fmt" is a character vector extracted by findVariableLists, and reflects how the data is formatted
+  # in the variable list.  "mode" is a length one string derived from fmt and reflects the processing
+  # needed, in particular which parameters need to be extracted in order to check the data
 
   fmt <- VL[["DataGuide"]][, "Format"][-1]
-
-  # "mode" is a length one string derived from fmt and is used to determine the type of
-  # processing needed; we are not using "fmt" because we need a more user-friendly string
-
   mode <- NA_character_
   if ("XYY" %in% fmt) mode <- "XY_data"
-  if ("XRR" %in% fmt) mode <- "NMR_1D"
+  if ("XRR" %in% fmt) mode <- "NMR_1D" # these files also contain XII
   if ("NMR_2D" %in% fmt) mode <- "NMR_2D"
+  if ("LC_MS" %in% fmt) mode <- "LC_MS"
   if ("PEAK_TABLE" %in% fmt) mode <- "PEAK_TABLE"
   if (is.na(mode)) stop("Could not determine the type of data in the file")
 
@@ -228,7 +226,7 @@ readJDX <- function(file = "", SOFC = TRUE, debug = 0) {
 
   params <- extractParams(VL[[2]], mode, SOFC, debug)
 
-  ##### Step 4.  Process the variable list(s) into the final lists
+  ##### Step 4.  Process the variable list(s) into the final list that is returned
 
   if ((mode == "XY_data") | (mode == "NMR_1D")) {
     # Return value is a list: dataGuide, metadata, comment lines + data frames of x, y
@@ -240,7 +238,7 @@ readJDX <- function(file = "", SOFC = TRUE, debug = 0) {
 
     # Fix up names
     if (mode == "XY_data") {
-      specnames <- jdx[blocks] # each line with title
+      specnames <- jdx[blocks] # each line with ##TITLE= (there is only one however)
       specnames <- str_trim(substring(specnames, 9, nchar(specnames)))
     }
 
@@ -262,17 +260,32 @@ readJDX <- function(file = "", SOFC = TRUE, debug = 0) {
     # Update VL
     VL[[4]] <- sort(seq(params[4], params[6], length.out = params[2])) # add F2
     VL[[5]] <- sort(seq(params[3], params[5], length.out = params[1])) # add F1
-    M <- M[nrow(M):1, ] # reverse order of rows, works for Bruker files
+    M <- M[nrow(M):1, ] # reverse order of rows, works for Bruker files (all vendors?)
     VL[[6]] <- M
     VL <- VL[1:6] # toss the other stuff
     names(VL) <- c("dataGuide", "metadata", "commentLines", "F2", "F1", "Matrix")
   }
 
+  if (mode == "LC_MS") {
+    # Return value is a list: dataGuide, metadata, comment lines, a data frame for each time point
+    # dataGuide, metadata & comments already in place; add data frames for each time point
+    for (i in 4:length(VL)) {
+      VL[[i]] <- processVariableList(VL[[i]], params, mode, VL[[1]][i - 2, c(2, 3)], SOFC, debug)
+      
+    }
+
+    # Get the retention times
+    rti <- grep("##PAGE= T=", jdx)
+    rt <- jdx[rti]
+    rt <- sub("##PAGE= ", "", rt)
+    names(VL) <- c("dataGuide", "metadata", "commentLines", rt)
+  }
+
   if (mode == "PEAK_TABLE") {
     for (i in 4:length(VL)) {
       VL[[i]] <- processVariableList(VL[[i]], params, mode, VL[[1]][i - 2, c(2, 3)], SOFC, debug)
     }
-    specnames <- jdx[blocks] # each line with title
+    specnames <- jdx[blocks] # each line with ##TITLE= (there is only one however)
     specnames <- str_trim(substring(specnames, 9, nchar(specnames)))
     names(VL) <- c("dataGuide", "metadata", "commentLines", specnames)
   }
